@@ -369,20 +369,22 @@ out_of_loop:
 int wait_for_job(size_t jobnumber, bool return_on_stop,
 	bool interruptible, bool return_on_trap)
 {
+    int signum = 0;
     job_T *job = joblist.contents[jobnumber];
+    bool savenonotify = job->j_nonotify;
 
+    job->j_nonotify = true;
     if (job->j_pgid >= 0 && job->j_status != JS_DONE
 	    && (!return_on_stop || job->j_status != JS_STOPPED)) {
 	block_sigchld_and_sigint();
 	do {
-	    int signum = wait_for_sigchld(interruptible, return_on_trap);
-	    if (signum)
-		return signum;
-	} while (job->j_status != JS_DONE
-		&& (!return_on_stop || job->j_status != JS_STOPPED));
+	    signum = wait_for_sigchld(interruptible, return_on_trap);
+	} while (signum == 0 && job->j_status != JS_DONE &&
+		(!return_on_stop || job->j_status != JS_STOPPED));
 	unblock_sigchld_and_sigint();
     }
-    return 0;
+    job->j_nonotify = savenonotify;
+    return signum;
 }
 
 /* Waits for a child process to finish (or stop).
@@ -406,7 +408,7 @@ wchar_t **wait_for_child(pid_t cpid, pid_t cpgid, bool return_on_stop)
     job->j_pgid = cpgid;
     job->j_status = JS_RUNNING;
     job->j_statuschanged = false;
-    job->j_loop = false;
+    job->j_nonotify = false;
     job->j_pcount = 1;
     job->j_procs[0].pr_pid = cpid;
     job->j_procs[0].pr_status = JS_RUNNING;
@@ -476,7 +478,7 @@ int calc_status(int status)
 	return WSTOPSIG(status) + TERMSIGOFFSET;
 #ifdef WIFCONTINUED
     if (WIFCONTINUED(status))
-	return 0;
+	return Exit_SUCCESS;
 #endif
     assert(false);
 }
@@ -503,19 +505,16 @@ int calc_status_of_job(const job_T *job)
 }
 
 /* Returns the name of the specified job.
- * If the job has only one process and is not a loop pipe,
- * `job->j_procs[0].pr_name' is returned.
+ * If the job has only one process, `job->j_procs[0].pr_name' is returned.
  * Otherwise, the names of all the process are concatenated and returned, which
  * must be freed by the caller. */
 wchar_t *get_job_name(const job_T *job)
 {
-    if (job->j_pcount == 1 && !job->j_loop)
+    if (job->j_pcount == 1)
 	return job->j_procs[0].pr_name;
 
     xwcsbuf_T buf;
     wb_init(&buf);
-    if (job->j_loop)
-	wb_cat(&buf, L"| ");
     for (size_t i = 0; i < job->j_pcount; i++) {
 	if (i > 0)
 	    wb_cat(&buf, L" | ");
@@ -595,21 +594,14 @@ char *get_job_status_string(const job_T *job, bool *needfree)
 
 /* Prints the status of job(s).
  * Finished jobs are removed from the job list after the status is printed.
- * If `jobnumber' is PJS_ALL, all the jobs are printed. If the specified job
- * doesn't exist, nothing is printed (it isn't an error).
+ * If the specified job doesn't exist, nothing is printed (it isn't an error).
  * If `changedonly' is true, only jobs whose `j_statuschanged' is true is
  * printed. If `verbose' is true, the status is printed in the process-wise
  * format rather than the usual job-wise format. */
 void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
 {
-    if (jobnumber == PJS_ALL) {
-	for (size_t i = 1; i < joblist.length; i++)
-	    print_job_status(i, changedonly, verbose, f);
-	return;
-    }
-
     job_T *job = get_job(jobnumber);
-    if (!job || (changedonly && !job->j_statuschanged))
+    if (!job || (changedonly && !job->j_statuschanged) || job->j_nonotify)
 	return;
 
     char current;
@@ -638,14 +630,13 @@ void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
 	char *status = get_process_status_string(
 		&job->j_procs[posixly_correct ? job->j_pcount - 1 : 0],
 		&needfree);
-	char looppipe = job->j_loop ? '|' : ' ';
 	wchar_t *jobname = job->j_procs[0].pr_name;
 
 	/* TRANSLATORS: the translated format string can be different 
 	 * from the original only in the number of spaces. This is required
 	 * for POSIX compliance. */
-	fprintf(f, gt("[%zu] %c %5jd %-20s %c %ls\n"),
-		jobnumber, current, (intmax_t) pid, status, looppipe, jobname);
+	fprintf(f, gt("[%zu] %c %5jd %-20s   %ls\n"),
+		jobnumber, current, (intmax_t) pid, status, jobname);
 	if (needfree)
 	    free(status);
 
@@ -668,6 +659,13 @@ void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
 	remove_job(jobnumber);
 }
 
+/* Prints the status of all jobs. */
+void print_job_status_all(bool changedonly, bool verbose, FILE *f)
+{
+    for (size_t i = 1; i < joblist.length; i++)
+	print_job_status(i, changedonly, verbose, f);
+}
+
 /* If the shell is interactive and the specified job has been killed by a
  * signal other than SIGINT or SIGPIPE, prints a notification to stderr. */
 void notify_signaled_job(size_t jobnumber)
@@ -989,7 +987,8 @@ int continue_job(size_t jobnumber, job_T *job, bool fg)
 
     if (fg)
 	wait_for_job(jobnumber, true, false, false);
-    int status = (job->j_status == JS_RUNNING) ? 0 : calc_status_of_job(job);
+    int status = (job->j_status == JS_RUNNING)
+	? Exit_SUCCESS : calc_status_of_job(job);
     if (job->j_status == JS_DONE) {
 	notify_signaled_job(jobnumber);
 	remove_job(jobnumber);
