@@ -45,7 +45,7 @@ typedef struct job_T {
     pid_t             j_pgid;          /* process group ID */
     enum jobstatus_T  j_status;
     _Bool             j_statuschanged; /* job's status not yet reported? */
-    _Bool             j_loop;          /* loop pipe? */
+    _Bool             j_nonotify;      /* supress printing job status? */
     size_t            j_pcount;        /* # of processes in `j_procs' */
     struct process_T  j_procs[];       /* info about processes */
 } job_T;
@@ -89,10 +89,11 @@ extern void put_foreground(pid_t pgrp);
 extern int calc_status_of_job(const job_T *job)
     __attribute__((pure,nonnull));
 
-#define PJS_ALL 0  /* stands for all the jobs */
 extern void print_job_status(
 	size_t jobnumber, _Bool changedonly, _Bool verbose, FILE *f)
     __attribute__((nonnull));
+extern void print_job_status_all(_Bool changedonly, _Bool verbose, FILE *f)
+    __attribute__((nonnull));
 extern void notify_signaled_job(size_t jobnumber);
 
 extern int jobs_builtin(int argc, void **argv)
