@@ -202,7 +202,8 @@ void redirsfree(redir_T *r)
     while (r) {
 	switch (r->rd_type) {
 	    case RT_INPUT:  case RT_OUTPUT:  case RT_CLOBBER:  case RT_APPEND:
-	    case RT_INOUT:  case RT_DUPIN:   case RT_DUPOUT:
+	    case RT_INOUT:  case RT_DUPIN:   case RT_DUPOUT:   case RT_PIPE:
+	    case RT_HERESTR:
 		wordfree(r->rd_filename);
 		break;
 	    case RT_HERE:  case RT_HERERT:
@@ -880,16 +881,6 @@ pipeline_T *parse_pipeline(void)
 	result->pl_neg = false;
     }
 
-    ensure_buffer(2);
-    if (!posixly_correct && cbuf.contents[cindex    ] == L'|'
-			 && cbuf.contents[cindex + 1] != L'|') {
-	result->pl_loop = true;
-	cindex++;
-	skip_blanks_and_comment();
-    } else {
-	result->pl_loop = false;
-    }
-
     result->pl_commands = parse_commands_in_pipeline();
     return result;
 }
@@ -1043,9 +1034,19 @@ void **parse_words_and_redirects(redir_T **redirlastp, bool first)
  * `*redirlastp' must be initialized to NULL beforehand. */
 void parse_redirect_list(redir_T **lastp)
 {
-    redir_T *redir;
+    for (;;) {
+#if YASH_ENABLE_ALIAS
+	if (!posixly_correct) {
+	    wchar_t *nameend;
+	    while (*(nameend = skip_name(cbuf.contents + cindex)) == L'\0'
+		    && read_more_input() == 0);
+	    substitute_alias(&cbuf, cindex, caliases, true);
+	}
+#endif
 
-    while ((redir = tryparse_redirect())) {
+	redir_T *redir = tryparse_redirect();
+	if (!redir)
+	    break;
 	*lastp = redir;
 	lastp = &redir->next;
     }
@@ -1147,9 +1148,11 @@ reparse:
 	switch (cbuf.contents[cindex + 1]) {
 	case L'<':
 	    if (cbuf.contents[cindex + 2] == L'-') {
-		result->rd_type = RT_HERERT; cindex += 3;
+		result->rd_type = RT_HERERT;  cindex += 3;
+	    } else if (!posixly_correct && cbuf.contents[cindex + 2] == L'<') {
+		result->rd_type = RT_HERESTR; cindex += 3;
 	    } else {
-		result->rd_type = RT_HERE;   cindex += 2;
+		result->rd_type = RT_HERE;    cindex += 2;
 	    }
 	    break;
 	case L'(':
@@ -1172,7 +1175,13 @@ reparse:
 		result->rd_type = RT_OUTPUT;  cindex += 1;
 	    }
 	    break;
-	case L'>':  result->rd_type = RT_APPEND;  cindex += 2;  break;
+	case L'>':
+	    if (!posixly_correct && cbuf.contents[cindex + 2] == L'|') {
+		result->rd_type = RT_PIPE;    cindex += 3;
+	    } else {
+		result->rd_type = RT_APPEND;  cindex += 2;
+	    }
+	    break;
 	case L'|':  result->rd_type = RT_CLOBBER; cindex += 2;  break;
 	case L'&':  result->rd_type = RT_DUPOUT;  cindex += 2;  break;
 	default:    result->rd_type = RT_OUTPUT;  cindex += 1;  break;
@@ -1517,7 +1526,8 @@ make_name:
     case L'}':
 	pe->pe_type |= PT_NONE;
 	if (pe->pe_type & PT_COLON)
-	    serror(Ngt("invalid use of `:' in parameter expansion"));
+	    serror(Ngt("invalid use of `%lc' in parameter expansion"),
+		    (wint_t) L':');
 	goto check_closing_paren_and_finish;
     case L'\0':  case L'\n':
 	serror(Ngt("`%ls' missing"), L"}");
@@ -1533,7 +1543,8 @@ parse_match:
 	if ((pe->pe_type & PT_MASK) == PT_SUBST)
 	    pe->pe_type |= PT_MATCHHEAD | PT_MATCHTAIL;
 	else
-	    serror(Ngt("invalid use of `:' in parameter expansion"));
+	    serror(Ngt("invalid use of `%lc' in parameter expansion"),
+		    (wint_t) L':');
 	cindex += 1;
     } else if (cbuf.contents[cindex] == cbuf.contents[cindex + 1]) {
 	if ((pe->pe_type & PT_MASK) == PT_MATCH)
@@ -1575,7 +1586,8 @@ check_closing_paren_and_finish:
     else
 	serror(Ngt("`%ls' missing"), L"}");
     if ((pe->pe_type & PT_NUMBER) && (pe->pe_type & PT_MASK) != PT_NONE)
-	serror(Ngt("invalid use of `#' flag in parameter expansion"));
+	serror(Ngt("invalid use of `%lc' in parameter expansion"),
+		(wint_t) L'#');
 
     wordunit_T *result = xmalloc(sizeof *result);
     result->next = NULL;
@@ -1824,7 +1836,7 @@ command_T *parse_group(commandtype_T type)
     result->c_lineno = cinfo->lineno;
     result->c_redirs = NULL;
     result->c_subcmds = parse_compound_list();
-    if (!result->c_subcmds)
+    if (posixly_correct && !result->c_subcmds)
 	serror(Ngt("no commands in command group"));
     if (cbuf.contents[cindex] == terminator[0])
 	cindex++;
@@ -1855,8 +1867,9 @@ command_T *parse_if(void)
 	ic->next = NULL;
 	if (!els) {
 	    ic->ic_condition = parse_compound_list();
-	    if (!ic->ic_condition)
-		serror(Ngt("no commands between `if' and `then'"));
+	    if (posixly_correct && !ic->ic_condition)
+		serror(Ngt("no commands between `%ls' and `%ls'"),
+			L"if", L"then");
 	    ensure_buffer(5);
 	    if (is_token_at(L"then", cindex))
 		cindex += 4;
@@ -1866,7 +1879,7 @@ command_T *parse_if(void)
 	    ic->ic_condition = NULL;
 	}
 	ic->ic_commands = parse_compound_list();
-	if (!ic->ic_commands)
+	if (posixly_correct && !ic->ic_commands)
 	    serror(Ngt("no commands after `%ls'"), els ? L"else" : L"then");
 	ensure_buffer(5);
 	if (!els) {
@@ -1927,11 +1940,17 @@ command_T *parse_for(void)
 	    serror(Ngt("redirection not allowed after `in'"));
 	    redirsfree(redirs);
 	}
+	if (cbuf.contents[cindex] == L';')
+	    cindex++;
     } else {
 	result->c_forwords = NULL;
+	if (cbuf.contents[cindex] == L';') {
+	    cindex++;
+	    if (posixly_correct)
+		serror(Ngt("`;' not allowed "
+			    "just after identifier in for loop"));
+	}
     }
-    if (cbuf.contents[cindex] == L';')
-	cindex++;
     skip_to_next_token();
     ensure_buffer(3);
     if (is_token_at(L"do", cindex))
@@ -1939,8 +1958,8 @@ command_T *parse_for(void)
     else
 	print_errmsg_token_missing(L"do", cindex);
     result->c_forcmds = parse_compound_list();
-    if (!result->c_forcmds)
-	serror(Ngt("no commands between `do' and `done'"));
+    if (posixly_correct && !result->c_forcmds)
+	serror(Ngt("no commands between `%ls' and `%ls'"), L"do", L"done");
     ensure_buffer(5);
     if (is_token_at(L"done", cindex))
 	cindex += 4;
@@ -1963,7 +1982,7 @@ command_T *parse_while(bool whltype)
     result->c_redirs = NULL;
     result->c_whltype = whltype;
     result->c_whlcond = parse_compound_list();
-    if (!result->c_whlcond)
+    if (posixly_correct && !result->c_whlcond)
 	serror(Ngt("no commands after `%ls'"), whltype ? L"while" : L"until");
     ensure_buffer(3);
     if (is_token_at(L"do", cindex))
@@ -1971,8 +1990,8 @@ command_T *parse_while(bool whltype)
     else
 	print_errmsg_token_missing(L"do", cindex);
     result->c_whlcmds = parse_compound_list();
-    if (!result->c_whlcmds)
-	serror(Ngt("no commands between `do' and `done'"));
+    if (posixly_correct && !result->c_whlcmds)
+	serror(Ngt("no commands between `%ls' and `%ls'"), L"do", L"done");
     ensure_buffer(5);
     if (is_token_at(L"done", cindex))
 	cindex += 4;
@@ -2459,8 +2478,6 @@ void print_pipelines(xwcsbuf_T *restrict buf, const pipeline_T *restrict p)
 	    wb_cat(buf, p->pl_cond ? L"&& " : L"|| ");
 	if (p->pl_neg)
 	    wb_cat(buf, L"! ");
-	if (p->pl_loop)
-	    wb_cat(buf, L"| ");
 	print_commands(buf, p->pl_commands);
     }
 }
@@ -2610,8 +2627,10 @@ void print_redirs(xwcsbuf_T *restrict buf, const redir_T *restrict r)
 	    case RT_INOUT:    s = L"<>";   type = file;  break;
 	    case RT_DUPIN:    s = L"<&";   type = file;  break;
 	    case RT_DUPOUT:   s = L">&";   type = file;  break;
+	    case RT_PIPE:     s = L">>|";  type = file;  break;
 	    case RT_HERE:     s = L"<<";   type = here;  break;
 	    case RT_HERERT:   s = L"<<-";  type = here;  break;
+	    case RT_HERESTR:  s = L"<<<";  type = file;  break;
 	    case RT_PROCIN:   s = L"<(";   type = proc;  break;
 	    case RT_PROCOUT:  s = L">(";   type = proc;  break;
 	    default: assert(false);
