@@ -577,7 +577,7 @@ void handle_sigchld(void)
         sigchld_received = false;
         do_wait();
 	if (shopt_notify) {
-	    print_job_status(PJS_ALL, true, false, stderr);
+	    print_job_status_all(true, false, stderr);
 	    // XXX redisplay prompt if lineedit is active
 	}
     }
@@ -831,7 +831,7 @@ void clear_traps(void)
 	rtsignal_received[i] = false;
     }
 #endif
-    any_trap_set = any_signal_received = false;
+    any_trap_set = false, any_signal_received = false;
 }
 
 
